#!/usr/bin/env python3

import datetime as dt
from pytz import timezone
from skyfield import almanac
from skyfield.api import N, W, wgs84, load
import time as time
import subprocess as subprocess
from PIL import Image, ImageFont, ImageDraw, ImageOps
import os
import cv2 as cv
import syslog
import numpy as np
import gc
import smtplib
import ephem
import send_sms

def camera_set(cap,mode):
    global fourcc, fps,height,width,exposure,gain,auto,auto_after,frameFrequency
    if mode == 'day':
        fourcc_cap = cv.VideoWriter_fourcc(*'MJPG')
        ret = cap.set(cv.CAP_PROP_FOURCC,fourcc_cap)
        fourcc = cap.get(cv.CAP_PROP_FOURCC)
        ret = cap.set(cv.CAP_PROP_FPS,30)
        fps = cap.get(cv.CAP_PROP_FPS)
        ret = cap.set(cv.CAP_PROP_FRAME_WIDTH,1920)
        ret = cap.set(cv.CAP_PROP_FRAME_HEIGHT,1080)
        height = cap.get(cv.CAP_PROP_FRAME_HEIGHT)
        width = cap.get(cv.CAP_PROP_FRAME_WIDTH)
        ret = cap.set(cv.CAP_PROP_GAIN,0)
        gain = cap.get(cv.CAP_PROP_GAIN)
        auto = cap.get(cv.CAP_PROP_AUTO_EXPOSURE)
        ret = cap.set(cv.CAP_PROP_AUTO_EXPOSURE,3)
        auto_after = cap.get(cv.CAP_PROP_AUTO_EXPOSURE)
        #Capture images per 3 frame
        frameFrequency = 3
        total_frame = 0
        exposure = "auto"
        gain = "auto"
    if mode == "night":
        fourcc_cap = cv.VideoWriter_fourcc(*'MJPG')
        ret = cap.set(cv.CAP_PROP_FOURCC,fourcc_cap)
        fourcc = cap.get(cv.CAP_PROP_FOURCC)
        ret = cap.set(cv.CAP_PROP_FPS,30)
        fps = cap.get(cv.CAP_PROP_FPS)
        ret = cap.set(cv.CAP_PROP_FRAME_WIDTH,1920)
        ret = cap.set(cv.CAP_PROP_FRAME_HEIGHT,1080)
        height = cap.get(cv.CAP_PROP_FRAME_HEIGHT)
        width = cap.get(cv.CAP_PROP_FRAME_WIDTH)
        auto = cap.get(cv.CAP_PROP_AUTO_EXPOSURE)
        ret = cap.set(cv.CAP_PROP_AUTO_EXPOSURE,1)
        #ret = cap.set(cv.CAP_PROP_AUTO_EXPOSURE,3)
        auto_after = cap.get(cv.CAP_PROP_AUTO_EXPOSURE)
        #ret = cap.set(cv.CAP_PROP_GAIN,50)
        ret = cap.set(cv.CAP_PROP_GAIN,90)
        gain = cap.get(cv.CAP_PROP_GAIN)
        ret = cap.set(cv.CAP_PROP_EXPOSURE,5000)
        exposure = cap.get(cv.CAP_PROP_EXPOSURE)
        frameFrequency = 20
        total_frame = 0
    if mode == "darks":
        fourcc_cap = cv.VideoWriter_fourcc(*'MJPG')
        ret = cap.set(cv.CAP_PROP_FOURCC,fourcc_cap)
        fourcc = cap.get(cv.CAP_PROP_FOURCC)
        ret = cap.set(cv.CAP_PROP_FPS,30)
        fps = cap.get(cv.CAP_PROP_FPS)
        ret = cap.set(cv.CAP_PROP_FRAME_WIDTH,1920)
        ret = cap.set(cv.CAP_PROP_FRAME_HEIGHT,1080)
        height = cap.get(cv.CAP_PROP_FRAME_HEIGHT)
        width = cap.get(cv.CAP_PROP_FRAME_WIDTH)
        auto = cap.get(cv.CAP_PROP_AUTO_EXPOSURE)
        ret = cap.set(cv.CAP_PROP_AUTO_EXPOSURE,1)
        auto_after = cap.get(cv.CAP_PROP_AUTO_EXPOSURE)
        ret = cap.set(cv.CAP_PROP_GAIN,70)
        gain = cap.get(cv.CAP_PROP_GAIN)
        ret = cap.set(cv.CAP_PROP_EXPOSURE,5000)
        exposure = cap.get(cv.CAP_PROP_EXPOSURE)
        frameFrequency = 10
        total_frame = 0
    return fourcc, fps,height,width,exposure,gain,auto,auto_after,frameFrequency    
def logging(comment):
    process_name = 'aurora_cam'
    syslog.openlog(process_name,logoption=syslog.LOG_PID, facility=syslog.LOG_LOCAL0)
    syslog.syslog("%s,%s" % (process_name,comment))
    syslog.closelog()
    print(comment)
    return

def add_border(input_image, border):
    img = Image.open(input_image)
    if isinstance(border, int) or isinstance(border, tuple):
        bimg = ImageOps.expand(img, border=border)
    else:
        raise RuntimeError('Border is not an integer or tuple!')
    bimg.save(input_image,'jpeg',quality=95)

def print_free_memory(comment):
    free = subprocess.check_output("free -m",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
    logging(comment +"\n" +  free)


def human_moon(observer):
    target_date_utc = observer.date
    target_date_local = ephem.localtime( target_date_utc ).date()
    next_full = ephem.localtime( ephem.next_full_moon(target_date_utc) ).date()
    next_new = ephem.localtime( ephem.next_new_moon(target_date_utc) ).date()
    next_last_quarter = ephem.localtime( ephem.next_last_quarter_moon(target_date_utc) ).date()
    next_first_quarter = ephem.localtime( ephem.next_first_quarter_moon(target_date_utc) ).date()
    previous_full = ephem.localtime( ephem.previous_full_moon(target_date_utc) ).date()
    previous_new = ephem.localtime( ephem.previous_new_moon(target_date_utc) ).date()
    previous_last_quarter = ephem.localtime( ephem.previous_last_quarter_moon(target_date_utc) ).date()
    previous_first_quarter = ephem.localtime( ephem.previous_first_quarter_moon(target_date_utc) ).date()
    if target_date_local in (next_full, previous_full):
        return 'Full'
    elif target_date_local in (next_new, previous_new):
        return 'New'
    elif target_date_local in (next_first_quarter, previous_first_quarter):
        return 'First Quarter'
    elif target_date_local in (next_last_quarter, previous_last_quarter):
        return 'Last Quarter'
    elif previous_new < next_first_quarter < next_full < next_last_quarter < next_new:
        return 'Waxing Crescent'
    elif previous_first_quarter < next_full < next_last_quarter < next_new < next_first_quarter:
        return 'Waxing Gibbous'
    elif previous_full < next_last_quarter < next_new < next_first_quarter < next_full:
        return 'Waning Gibbous'
    elif previous_last_quarter < next_new < next_first_quarter < next_full < next_last_quarter:
        return 'Waning Crescent'

aurora_string = ''
#only run the aurora_cam after twilight begins in the evening through the end of twilight in the morning
zone = timezone('US/Mountain')
now = zone.localize(dt.datetime.now())
logging("Starting new aurora_cam process...%s" % (now))
current_hour = now.hour
if current_hour < 12:
    now = now - dt.timedelta(days=1)
    midnight = now.replace(hour=0, minute=0, second=0, microsecond=0)
    next_midnight = midnight + dt.timedelta(days=1)
    second_midnight = midnight + dt.timedelta(days=2)
else:
    midnight = now.replace(hour=0, minute=0, second=0, microsecond=0)
    next_midnight = midnight + dt.timedelta(days=1)
    second_midnight = midnight + dt.timedelta(days=2)

logging("now = %s" % (now))


ts = load.timescale()
t0 = ts.from_datetime(midnight)
t1 = ts.from_datetime(next_midnight)
t2 = ts.from_datetime(second_midnight)
print(t0,t1,t2)
eph = load('de421.bsp')
moon, earth = eph['moon'], eph['earth']
starhouse = wgs84.latlon(46.38611 * N, 111.40983 * W)
f = almanac.dark_twilight_day(eph, starhouse)
print("Calculating for t0, t1")
times, events = almanac.find_discrete(t0, t1, f)
for t, e in zip(times, events):
    tstr = str(t.astimezone(zone))
    hour = int(tstr[:-16].split(' ')[1].split(':')[0])
    if hour > 12 and e == 1:
        #nautical twilight
        begin_evening_twilight = tstr
print("Calculating for t1,t2")
times, events = almanac.find_discrete(t1, t2, f)
for t, e in zip(times, events):
    tstr = str(t.astimezone(zone))
    hour = int(tstr[:-16].split(' ')[1].split(':')[0])
    if hour < 12 and e == 2:
        end_morning_twilight = tstr
    print(e, tstr, ' ', almanac.TWILIGHTS[e])
logging("Begin Twilight = %s" % (begin_evening_twilight))
logging("End Twilight = %s" % (end_morning_twilight))
day_or_night_query = "select case when clock_timestamp() between ('%s'::timestamp with time zone) and ('%s'::timestamp with time zone) then 'night' else 'day' end as day_or_night" % (begin_evening_twilight,end_morning_twilight)
dir_date_query = "select case when (clock_timestamp() between ('%s'::timestamp with time zone) and ('%s'::timestamp with time zone) and extract (hour from clock_timestamp()) between 0 and 8) then (current_date - 1) else current_date end as dir_date" % (begin_evening_twilight,end_morning_twilight)
day_or_night = subprocess.check_output("psql -h 10.0.26.1 -X -A -t -q -d postgres  -c \"%s\"" % (day_or_night_query),stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
y = 0
while y == 0:
    video_id = subprocess.check_output("v4l2-ctl --list-devices | grep -A1 \"2.0\" | grep dev | cut -f2 -d \"o\"",stderr=subprocess.STDOUT, shell=True).decode('utf-8')
    #video_id = subprocess.check_output("usbdevs -v | grep -A1 \"ff.ff\" | grep uvideo | cut -f2 -d \"o\"",stderr=subprocess.STDOUT, shell=True).decode('utf-8')
    if int(video_id) == 0:
        cap = cv.VideoCapture(cv.CAP_V4L2)
    else:
        cap = cv.VideoCapture(int(video_id), cv.CAP_V4L2)

    if not cap.isOpened():
        logging("capture opening failed...sleeping 3 seconds")
        try:
            pid = subprocess.check_output("fuser -k %s" % (video_device),stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
        except:
            pass
        time.sleep(3)
        y = 0
    else:
        logging("capture is open...")
        camera_set(cap,day_or_night)
        logging("%s,%s,%s,%s,%s,%s,%s,%s,%s" % (fourcc, fps,height,width,exposure,gain,auto,auto_after,frameFrequency))
        y = 1
while y:
    starhouse = ephem.Observer()
    starhouse.lat = '46.38611'
    starhouse.lon = '-111.40983'
    starhouse.elevation = 1268
    m = ephem.Moon()
    aurora_string = ''
    start = time.time()
    current_time = subprocess.check_output("psql -X -h 10.0.26.1 -A -t -q -d postgres  -c \"select clock_timestamp()\"",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
    ephem_current_time = subprocess.check_output("psql -X -h 10.0.26.1 -A -t -q -d postgres -c \"select date_trunc('second',now() at time zone 'UTC')\"",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
    print(ephem_current_time)
    m.compute(ephem_current_time)
    starhouse.date = ephem_current_time
    current = m.compute(starhouse)
    logging("Start time this run: %s" %  (current_time))
    logging(day_or_night_query)
    day_or_night = subprocess.check_output("psql -X -h 10.0.26.1 -A -t -q -d postgres  -c \"%s\"" % (day_or_night_query),stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
    dir_date = subprocess.check_output("psql -X -h 10.0.26.1 -A -t -q -d postgres  -c \"%s\"" % (dir_date_query),stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
    logging(day_or_night + ' ' +  dir_date)
    subprocess.check_output("mkdir -p /starhouse/aurora_cam/%s" % (dir_date),stderr=subprocess.STDOUT, shell=True)
    logging("dir_date = %s" % (dir_date))
    #day_or_night = "night"
    #day_or_night = "darks"
    #exit()
    camera_set(cap,day_or_night)
    logging("%s,%s,%s,%s,%s,%s,%s,%s,%s" % (fourcc, fps,height,width,exposure,gain,auto,auto_after,frameFrequency))
    if day_or_night == "darks":
        logging("darks")
        z = 0
        frame = []
        epoch_filename = []
        total_frame = 0
        logging("Elapsed time before capturing frames: %s" % (time.time() - start))
        #set_camera = subprocess.check_output("v4l2-ctl -d0 --try-fmt-video width=1920,height=1080,pixelformat=0",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
        #set_exposure = subprocess.check_output("v4l2-ctl -d0 -c exposure_auto=3,exposure_absolute=5000,gain=70",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
        while z < frameFrequency:
            try:
                ret, temp_frame = cap.read()
                if ret is False:
                    break
                frame.append(temp_frame)
                logging("Elapsed time after snapshot #%s: %s" % (z,time.time() - start))
                total_frame += 1
            except Exception as e:
                logging(e)
                #kill anything else using the camera
                #pid = subprocess.check_output("fuser -k /dev/video0",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
                logging("Exception caught! %s" % (e))
            z += 1
        logging("Elapsed time after capturing %s images: %s" % (z,time.time() - start))

        #we now have frameFrequency images as a series of numpy arrays
        alpha = .3
        beta = (1 - alpha)
        a = 0
        dst = frame[0]
        dst32 = dst.astype(np.int32)
        while a <  len(frame):
            image_name = "/starhouse/aurora_cam/darks/%s" % (str(time.time()) + ".jpg")
            #cv.imwrite(image_name, frame[a])
            dst32 = cv.add(dst32,frame[a].astype(np.int32))
            a += 1
        dst32 = cv.multiply(dst32,(1.5,1.5,1.5,1.5),0.8)
        dst = cv.normalize(dst32,None,0,255,cv.NORM_MINMAX) #opencv normalization
        epoch_filename = "2K_spinel_dark_" + str(time.time()) + ".jpg"
        image_name = "/starhouse/aurora_cam/darks/%s" % (epoch_filename)
        cv.imwrite(image_name, dst)
        logging("Elapsed time after writing %s images: %s" % (len(epoch_filename),time.time() - start))
    
    if day_or_night == "day":
        break
        logging("day")
        print(ephem_current_time)
        print(ephem.previous_new_moon(ephem_current_time))
        print(ephem.next_full_moon(ephem_current_time))
        print(ephem.next_new_moon(ephem_current_time))
        print_free_memory("start")
        phase = human_moon(starhouse) + " " + str(round(m.phase,2))
        print(phase)

        subprocess.check_output("mkdir -p /starhouse/aurora_cam/day/%s" % (dir_date),stderr=subprocess.STDOUT, shell=True)
        print_free_memory("start")
        gc.collect()
        z = 0
        frame = []
        epoch_filename = []
        total_frame = 0
        logging("Elapsed time before capturing frames: %s" % (time.time() - start))
        while z < frameFrequency:
            #set_camera = subprocess.check_output("v4l2-ctl -d0 --set-fmt-video width=1920,height=1080,pixelformat=0",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
            #set_exposure = subprocess.check_output("v4l2-ctl -d0 -c exposure_auto=3,gain=0,power_line_frequency=2",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
            #get_values = subprocess.check_output("v4l2-ctl -d0 --get-fmt-video",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
            #print(get_values)
            ##codec = cv.VideoWriter_fourcc('H', '2', '6', '4')
            #cap.set(6,codec)
            #cap.set(5,30)
            #cap.set(3,1920)
            #cap.set(4,1080)

            try:
                ret, temp_frame = cap.read()
                if ret is False:
                    break
                frame.append(temp_frame)
                logging("Elapsed time after snapshot #%s: %s" % (z,time.time() - start))
                total_frame += 1
            except Exception as e:
                logging(e)
                #kill anything else using the camera
                #pid = subprocess.check_output("fuser -k /dev/video0",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
                logging("Exception caught! %s" % (e))
            z += 1
        logging("Elapsed time after capturing %s images: %s" % (z,time.time() - start))

        #we now have frameFrequency images as a series of numpy arrays
        alpha = .3
        beta = (1 - alpha)
        a = 0
        dst = frame[0]
        dst32 = dst.astype(np.int32)
        while a <  len(frame):
            image_name = "/starhouse/aurora_cam/day/%s/%s" % (dir_date,str(time.time()) + ".jpg")
            cv.imwrite(image_name, frame[a])
            dst32 = cv.add(dst32,frame[a].astype(np.int32))
            a += 1
        dst = cv.normalize(dst32,None,0,255,cv.NORM_MINMAX) #opencv normalization
        epoch_filename = "2K_spinel_day_" + str(time.time()) + ".jpg"
        image_name = "/starhouse/aurora_cam/day/%s/%s" % (dir_date,epoch_filename)
        cv.imwrite(image_name, dst)
        logging("Elapsed time after writing %s images: %s" % (z,time.time() - start))

    if day_or_night == "night":
        logging("night")
        #set_camera = subprocess.check_output("v4l2-ctl -d0 --set-fmt-video width=1920,height=1080,pixelformat=0",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
        #set_exposure = subprocess.check_output("v4l2-ctl -d0 -c exposure_auto=3,exposure_absolute=5000,gain=70",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
        camera_set(cap,day_or_night)
        logging("%s,%s,%s,%s,%s,%s,%s,%s,%s" % (fourcc, fps,height,width,exposure,gain,auto,auto_after,frameFrequency))

        print(ephem_current_time)
        print(ephem.previous_new_moon(ephem_current_time))
        print(ephem.next_full_moon(ephem_current_time))
        print(ephem.next_new_moon(ephem_current_time))
        print_free_memory("start")
        phase = human_moon(starhouse) + " " + str(round(m.phase,2))
        print(phase)
        gc.collect()
        z = 0
        frame = []
        epoch_filename = []
        total_frame = 0
        dark_file = "/home/jross/2K_dark.jpg"
        logging("Elapsed time before capturing frames: %s" % (time.time() - start))
        while z < frameFrequency:
            print_free_memory("capturing %s frames" % (z))
            try:
                ret, temp_frame = cap.read()
                if ret is False:
                    break
                frame.append(temp_frame)
                logging("Elapsed time after snapshot #%s: %s" % (z,time.time() - start))
                total_frame += 1
            except Exception as e:
                logging(e)
                #kill anything else using the camera
                #pid = subprocess.check_output("fuser -k /dev/video0",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
                logging("Exception caught! %s" % (e))
            z += 1
        logging("Elapsed time after capturing %s images: %s" % (z,time.time() - start))
        print_free_memory("after all captures")
        gc.collect()

        #we now have frameFrequency images as a series of numpy arrays
        alpha = .3
        beta = (1 - alpha)
        a = 0
        dst = frame[0]
        dst32 = dst.astype(np.int32)
        while a <  len(frame):
            #image_name = "/starhouse/aurora_cam/%s/%s" % (dir_date,str(time.time()) + ".jpg")
            #cv.imwrite(image_name, frame[a])
            dst32 = cv.add(dst32,frame[a].astype(np.int32))
            a += 1
        frame = []
        print_free_memory("after averaging frames")    
        gc.collect()
        average_noise = cv.imread(dark_file).astype(np.int32)
        after_dark_removal = cv.absdiff(dst32, average_noise)
        dst32 = cv.multiply(after_dark_removal,(1.5,1.5,1.5,1.5),0.8)
        dst = cv.normalize(dst32,None,0,255,cv.NORM_MINMAX) #opencv normalization
        average = 0
        average_noise = 0
        dst32 = 0
        #dst[dst < 20.] = 0
        dst[dst < 10.] = 0
        epoch_filename = "2K_spinel_" + str(time.time()) + ".jpg"
        image_name = "/starhouse/aurora_cam/%s/%s" % (dir_date,epoch_filename)
        cv.imwrite(image_name, dst)
        logging("Elapsed time after writing %s image: %s" % (epoch_filename,time.time() - start))
        logging(dst.dtype)

        #convert to grayscale and check for colors
        red_boundary = [([17, 15, 100], [50, 56, 200])]
        green_boundary = [([29, 86, 6], [64, 255, 255])]
        dst = np.array(dst, dtype=np.uint8)
        for (lower, upper) in green_boundary:
            # create NumPy arrays from the boundaries
            lower = np.array(lower, dtype="uint8")
            upper = np.array(upper, dtype="uint8")
            # find the colors within the specified boundaries and apply the mask
            mask = cv.inRange(dst, lower, upper)
            output = cv.bitwise_and(dst, dst, mask=mask)
            output2 = cv.cvtColor(cv.bitwise_and(output, output, mask=mask), cv.COLOR_BGR2GRAY)
            green_hot = cv.countNonZero(output2)
            logging("Green hot = %s" % (green_hot))
            logging("Elapsed time after checking %s image for green: %s" % (epoch_filename,time.time() - start))
            if green_hot > 200:
                logging("aurora detected %s at %s" % (green_hot,str(time.time())))
                #send_sms("aurora")
        for (lower, upper) in red_boundary:
            # create NumPy arrays from the boundaries
            lower = np.array(lower, dtype="uint8")
            upper = np.array(upper, dtype="uint8")
            # find the colors within the specified boundaries and apply the mask
            mask = cv.inRange(dst, lower, upper)
            output = cv.cvtColor(cv.bitwise_and(dst, dst, mask=mask), cv.COLOR_BGR2GRAY)
            red_hot = cv.countNonZero(output)
            logging("Red hot = %s" % (red_hot))
            logging("Elapsed time after checking %s image for red: %s" % (epoch_filename,time.time() - start))
            if red_hot > 200:
                logging("fire detected %s at %s" % (red_hot,str(time.time())))
                #send_sms("fire")
        if green_hot > 200 and red_hot < 200:
            aurora_string = "Possible Aurora Detected:  Green: %s" % (green_hot)
        elif green_hot < 200 and red_hot > 200:
            aurora_string = "Possible Aurora Detected:  Red: %s" % (red_hot)
        elif green_hot > 200 and red_hot > 200:
            aurora_string = "Possible Aurora Detected:  Green: %s, Red: %s" % (green_hot,red_hot)
        else:
            aurora_string = ''
    current_time = time.time()
    logging("current_time = %s" % (current_time ))
    current_weather = []

    #columns returned from latest_weather.sql
    #timestamp | barometer | out_temp | humidity | wind | wind_dir_raw | wind_dir | wind_gust | wind_gust_dir_raw | wind_gust_dir | rain_rate | radiation
    #pi@allsky:
    #timestamp 0       | barometer 1| out_temp 2| humidity 3| wind 4| wind_dir_raw 5| wind_dir 6 | wind_gust 7 | wind_gust_dir_raw 8 | wind_gust_dir 9 | rain_rate 10 | radiation 11
    #------------------------+-----------+----------+----------+------+--------------+----------+-----------+-------------------+---------------+-----------+-----------
    #2020-03-01 11:38:00-07 |     30.14 |    33.10 |    72.50 | 6.15 |         7.90 | N        |      9.20 |            353.00 | N             |         0 |    464.57
    #y = y + 1
    logging("getting latest weather  %s" % (y))
    try:
        current_weather = subprocess.check_output("psql -X -h 10.0.26.1 -A -t -q  -d weather -f pg_latest_weather.sql",stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8').split("|")
    except Exception as e:
        logging("Weather exception caught! %s" % (e))
        b = 0
        while b < 12:
            current_weather.append('')
            b += 1
    current_time = time.time()
    logging("Elapsed time after getting weather: %s" % (current_time - start))
    logging(current_weather)
    barometer = current_weather[1]
    temp = current_weather[2] + u"\u00B0" + "F"
    humidity = current_weather[3]
    wind = current_weather[4]
    wind_dir = current_weather[5]
    wind_gust = current_weather[6]
    wind_gust_dir = current_weather[7]
    rain_rate = current_weather[8]
    solar = current_weather[9] + u" W/m\u00B2"
    ts = load.timescale()
    t = ts.now()
    moon_string = phase + "%, RA/Dec: " + str(m.ra).replace(':','h',1).replace(':','m') + "s " + str(m.dec).replace(':',u'\u00B0',1).replace(':','\'') + "\", Alt/Az: " + str(m.alt).replace(':',u'\u00B0',1).replace(':','\'') + "\", " + str(m.az).replace(':',u'\u00B0',1).replace(':','\'') + "\"  Mag: " + str(m.mag)
    moon_string = phase + "%, Alt/Az: " + str(m.alt).replace(':',u'\u00B0',1).replace(':','\'') + "\", " + str(m.az).replace(':',u'\u00B0',1).replace(':','\'') + "\"  Mag: " + str(m.mag)
    top_string="Starhouse Observatory - North View - Big Belts"
    weather_string = "Temp: %s,  Pressure: %sinHG, Humidity: %s%%, Wind: %s %s MPH Gust %s MPH" % (temp,barometer,humidity,wind_dir,wind,wind_gust)
    top_string="Starhouse Observatory - Moon: %s, Weather: %s" % (moon_string,weather_string)
    current_time = time.time()
    logging("Elapsed time before snapshot: %s" % (time.time() - start))

    if os.path.exists(image_name):
        logging(image_name)
        current_time = time.time()
        logging("Elapsed time after snapshot: %s" % (current_time - start))
        add_border(image_name,(45,25,45,25))
        img = Image.open("%s" % (image_name))
        draw = ImageDraw.Draw(img)
        font = ImageFont.truetype("/usr/share/fonts/truetype/msttcorefonts/Andale_Mono.ttf", 14)
        logging(epoch_filename)
        if day_or_night == 'day':
            secs_since_epoch = epoch_filename.split('_')[3].split('.')[0]
        else:
            secs_since_epoch = epoch_filename.split('_')[2].split('.')[0]
        timestamp = subprocess.check_output("psql -h 10.0.26.1 -d postgres  -A -t -q -X -c \"select to_timestamp(%s)\"" % (secs_since_epoch),stderr=subprocess.STDOUT, shell=True)[:-1].decode('utf-8')
        h_offset = 100
        v_offset = 5
        draw.text((h_offset, v_offset),top_string,(255,255,255),font=font)
        h_offset = 2010 - len(top_string) - 900
        h_offset = 1600 - len(top_string) 
        v_offset = 1110
        logging("h_offset: %s, v_offset: %s" % (h_offset,v_offset))
        string_to_write = "%s   %s" % (timestamp, epoch_filename)
        copyright = u'\u00a9 Jeff Ross, All Rights Reserved  2K Spinel UC20MPG_SL40 f1.4/4mm lens'
        draw.text((100,v_offset),copyright,(255,255,255),font=font)
        draw.text((h_offset,v_offset),string_to_write,(255,255,255),font=font)
        img.save(image_name,'jpeg',quality=95,optimize=True,subsampling=0)
        #check for correct image size
        img = Image.open("%s" % (image_name))
        w,h = img.size
        logging("width = %s, height = %s %s" % (w,h,image_name))
        if w != 2010  or h != 1130:
            logging("width = %s, height = %s--removing %s" % (w,h,image_name))
            os.remove(image_name)
        current_time = time.time()

        logging("Elapsed time after adding border and text: %s" % (current_time - start))
        current_time = time.time()
        logging("Total Elapsed Time: %s\n" % (current_time - start))
    else:
        logging("image_name %s not found on disk!" % (image_name))
    y = y + 1

